import React, { PureComponent } from 'react';
import { Platform, StyleSheet } from 'react-native';
import { TabViewAnimated, TabViewPagerAndroid, TabViewPagerScroll, TabViewPagerPan } from 'react-native-tab-view';
import TabBarTop from './TabBarTop';
import TabBarBottom from './TabBarBottom';
import SceneView from '../SceneView';
import withCachedChildNavigation from '../../withCachedChildNavigation';

var babelPluginFlowReactPropTypes_proptype_NavigationTabScreenOptions = require('../../TypeDefinition').babelPluginFlowReactPropTypes_proptype_NavigationTabScreenOptions || require('react').PropTypes.any;

var babelPluginFlowReactPropTypes_proptype_NavigationRouter = require('../../TypeDefinition').babelPluginFlowReactPropTypes_proptype_NavigationRouter || require('react').PropTypes.any;

var babelPluginFlowReactPropTypes_proptype_NavigationState = require('../../TypeDefinition').babelPluginFlowReactPropTypes_proptype_NavigationState || require('react').PropTypes.any;

var babelPluginFlowReactPropTypes_proptype_NavigationAction = require('../../TypeDefinition').babelPluginFlowReactPropTypes_proptype_NavigationAction || require('react').PropTypes.any;

var babelPluginFlowReactPropTypes_proptype_NavigationRoute = require('../../TypeDefinition').babelPluginFlowReactPropTypes_proptype_NavigationRoute || require('react').PropTypes.any;

var babelPluginFlowReactPropTypes_proptype_NavigationScreenProp = require('../../TypeDefinition').babelPluginFlowReactPropTypes_proptype_NavigationScreenProp || require('react').PropTypes.any;

if (typeof exports !== 'undefined') Object.defineProperty(exports, 'babelPluginFlowReactPropTypes_proptype_TabViewConfig', {
  value: require('react').PropTypes.shape({
    tabBarComponent: require('prop-types').any,
    tabBarPosition: require('prop-types').oneOf(['top', 'bottom']),
    tabBarOptions: require('prop-types').shape({}),
    swipeEnabled: require('prop-types').bool,
    animationEnabled: require('prop-types').bool,
    lazyLoad: require('prop-types').bool
  })
});
if (typeof exports !== 'undefined') Object.defineProperty(exports, 'babelPluginFlowReactPropTypes_proptype_TabScene', {
  value: require('react').PropTypes.shape({
    route: babelPluginFlowReactPropTypes_proptype_NavigationRoute,
    focused: require('prop-types').bool.isRequired,
    index: require('prop-types').number.isRequired,
    tintColor: require('prop-types').string
  })
});


let TabViewPager;

switch (Platform.OS) {
  case 'android':
    TabViewPager = TabViewPagerAndroid;
    break;
  case 'ios':
    TabViewPager = TabViewPagerScroll;
    break;
  default:
    TabViewPager = TabViewPagerPan;
}

class TabView extends PureComponent {

  static TabBarTop = TabBarTop;
  static TabBarBottom = TabBarBottom;

  _handlePageChanged = index => {
    const { navigation } = this.props;
    navigation.navigate(navigation.state.routes[index].routeName);
  };

  _renderScene = ({ route }) => {
    const { screenProps } = this.props;
    const childNavigation = this.props.childNavigationProps[route.key];
    const TabComponent = this.props.router.getComponentForRouteName(route.routeName);
    return <SceneView screenProps={screenProps} component={TabComponent} navigation={childNavigation} navigationOptions={this.props.router.getScreenOptions(childNavigation, screenProps)} />;
  };

  _getLabel = ({ focused, route, tintColor }) => {
    const options = this.props.router.getScreenOptions(this.props.childNavigationProps[route.key], this.props.screenProps || {});

    if (options.tabBarLabel) {
      return options.tabBarLabel;
    }

    if (typeof options.title === 'string') {
      return options.title;
    }

    return route.routeName;
  };

  _renderIcon = ({ focused, route, tintColor }) => {
    const options = this.props.router.getScreenOptions(this.props.childNavigationProps[route.key], this.props.screenProps || {});
    if (options.tabBarIcon) {
      return typeof options.tabBarIcon === 'function' ? options.tabBarIcon({ tintColor, focused }) : options.tabBarIcon;
    }
    return null;
  };

  _renderTabBar = props => {
    const {
      tabBarOptions,
      tabBarComponent: TabBarComponent,
      animationEnabled
    } = this.props;
    if (typeof TabBarComponent === 'undefined') {
      return null;
    }
    return <TabBarComponent {...props} {...tabBarOptions} navigation={this.props.navigation} getLabel={this._getLabel} renderIcon={this._renderIcon} animationEnabled={animationEnabled} />;
  };

  _renderPager = props => {
    const {
      swipeEnabled,
      animationEnabled
    } = this.props;

    return <TabViewPager {...props} swipeEnabled={swipeEnabled} animationEnabled={animationEnabled} />;
  };

  _configureTransition = () => null;

  render() {
    const {
      router,
      navigation,
      tabBarComponent,
      tabBarPosition,
      animationEnabled,
      lazyLoad,
      screenProps
    } = this.props;

    let renderHeader;
    let renderFooter;

    const { state } = this.props.navigation;
    const options = router.getScreenOptions(this.props.childNavigationProps[state.routes[state.index].key], screenProps || {});

    const tabBarVisible = options.tabBarVisible == null ? true : options.tabBarVisible;

    if (tabBarComponent !== undefined && tabBarVisible) {
      if (tabBarPosition === 'bottom') {
        renderFooter = this._renderTabBar;
      } else {
        renderHeader = this._renderTabBar;
      }
    }

    let configureTransition;

    if (animationEnabled === false) {
      configureTransition = this._configureTransition;
    }

    return (
      /* $FlowFixMe */
      <TabViewAnimated style={styles.container} navigationState={this.props.navigation.state} lazy={lazyLoad} renderHeader={renderHeader} renderFooter={renderFooter} renderScene={this._renderScene} renderPager={this._renderPager} configureTransition={configureTransition} onRequestChangeTab={this._handlePageChanged} screenProps={this.props.screenProps} />
    );
  }
}

TabView.propTypes = {
  tabBarComponent: require('prop-types').any,
  tabBarPosition: require('prop-types').oneOf(['top', 'bottom']),
  tabBarOptions: require('prop-types').shape({}),
  swipeEnabled: require('prop-types').bool,
  animationEnabled: require('prop-types').bool,
  lazyLoad: require('prop-types').bool,
  screenProps: require('prop-types').shape({}),
  navigation: babelPluginFlowReactPropTypes_proptype_NavigationScreenProp,
  router: babelPluginFlowReactPropTypes_proptype_NavigationRouter,
  childNavigationProps: require('prop-types').shape({}).isRequired
};
TabView.propTypes = {
  tabBarComponent: require('prop-types').any,
  tabBarPosition: require('prop-types').oneOf(['top', 'bottom']),
  tabBarOptions: require('prop-types').shape({}),
  swipeEnabled: require('prop-types').bool,
  animationEnabled: require('prop-types').bool,
  lazyLoad: require('prop-types').bool,
  screenProps: require('prop-types').shape({}),
  navigation: babelPluginFlowReactPropTypes_proptype_NavigationScreenProp,
  router: babelPluginFlowReactPropTypes_proptype_NavigationRouter,
  childNavigationProps: require('prop-types').shape({}).isRequired
};
const TabViewEnhanced = withCachedChildNavigation(TabView);

TabViewEnhanced.TabBarTop = TabBarTop;
TabViewEnhanced.TabBarBottom = TabBarBottom;

export default TabViewEnhanced;

const styles = StyleSheet.create({
  container: {
    flex: 1
  }
});